# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#

newproperty(:check) do
  include EasyType
  include PuppetX::EnterpriseModules::Db2::Property

  desc <<-DESC
  The query to execute as a check.
  DESC

  def change_to_s(_current, _should)
    'The check has executed and found issues.'
  end

  def insync?(_to)
    return true if resource[:refreshonly].to_s == 'true'

    if resource.skip.to_s == 'true'
      message = 'Skipping this check at users request.'
      Puppet.debug "#{path}: #{message}"
      resource.class.register_result(resource.control, 'SKIPPED', message)
      return true
    end

    records = provider.execute_check
    result = records.count == resource[:expected_number_of_rows]
    message = result ? resource.title : resource.fail_message

    resource.class.register_result(resource.control, result ? 'PASSED' : 'FAILED', message)
    resource.report_as.to_s.casecmp('none').zero? ? true : result
  end

  data_type('String[1]')
end
