# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet'
module_path = File.absolute_path("#{__dir__}/../../../../../")
$LOAD_PATH.unshift("#{module_path}/db2_config/lib")
$LOAD_PATH.unshift("#{module_path}/easy_type/lib")
require 'easy_type/preamble'
require 'easy_type/setup'
require 'easy_type'
require 'puppet_x/enterprisemodules/db2/access'

module PuppetX
  # Doc
  module Db2
    # Docs
    # TODO: Write documentation
    module Fact
      include PuppetX::EnterpriseModules::Db2::Type::ClassMethods

      def db2_array_fact_all(mappings = {}, &proc)
        query_routine = lambda { |query| sql_on_all_primary_databases(query) }
        mapper        = lambda { |results, lambda_mappings| array_mapper(results, lambda_mappings) }
        db2_fact_internal(query_routine, mapper, mappings, &proc)
      end

      def db2_array_fact(mappings = {}, &proc)
        query_routine = lambda { |query| sql_on_all_primary_databases(query) }
        mapper        = lambda { |results, lambda_mappings| array_mapper(results, lambda_mappings) }
        db2_fact_internal(query_routine, mapper, mappings, &proc)
      end

      def db2_record_fact(mappings = {}, &proc)
        query_routine = lambda { |query| sql_on_all_databases(query) }
        mapper        = lambda { |results, lambda_mappings| record_mapper(results, lambda_mappings) }
        db2_fact_internal(query_routine, mapper, mappings, &proc)
      end

      def db2_fact_internal(query_routine, mapper, mappings = {})
        query = yield
        setcode do
          extend PuppetX::EnterpriseModules::Db2::Access
          extend ::EasyType::Helpers

          #
          # If there is an error in the query we want to return null, meaning the
          # fact is not resolved
          #
          catch(:error_in_query) do
            results = query_routine.call(query)
            mapper.call(results, mappings)
          end
        end
      end

      def no_mappings(result)
        result.delete(:instance)
        result.delete(:database)
        result[result.keys.first]
      end

      def array_mapper(results, mappings)
        results.collect do |result|
          if mappings == {}
            no_mappings(result)
          else
            return_value = mappings.collect do |key, entry|
              [key.to_s, result[entry].strip]
            end.flatten
            Hash[*return_value]
          end
        end
      end

      def record_mapper(result, mappings)
        result = result.first
        if mappings == {}
          no_mappings(result)
        else
          return_value = mappings.collect do |key, entry|
            [key.to_s, result[entry].strip]
          end
          return_value.to_h
        end
      end
    end
  end
end
