# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
module PuppetX
  module EnterpriseModules
    module Db2
      # Add documentation TODO
      module GrantProperty
        def self.included(parent)
          parent.send(:include, ::EasyType)
          parent.send(:include, ::EasyType::Mungers::Upcase)
          parent.send(:include, PuppetX::EnterpriseModules::Db2::Commands)
          parent.send(:include, PuppetX::EnterpriseModules::Db2::Property)
          parent.extend(ClassMethods)
        end

        def primary?(instance, database)
          resource.class.primary_database_names_on(instance).include?(database)
        end

        def when_primary
          if primary?(resource.instance_name, resource.database_name)
            yield
          else
            Puppet.notice "#{resource.path}: Skipping DB2 query because on database #{resource.database_name} on instance #{resource.instance_name} is running in standby mode."
            nil
          end
        end

        def change_to_s(_from, _to)
          return_value = []
          return_value << "revoked the #{revoked_rights.join(',')} right(s)" if revoked_rights.any?
          return_value << "granted the #{granted_rights.join(',')} right(s)" if granted_rights.any?
          return_value.join(' and ')
        end

        def revoked_rights
          []
        end

        def granted_rights
          []
        end

        def current_rights
          return [] if provider.grants == :absent

          provider.grants
        end

        def revoke(rights)
          case self.class.to_s
          when /role/
            "revoke #{rights.join(',')} on database from role #{resource.role_name} by all"
          when /user/
            "revoke #{rights.join(',')} on database from user #{resource.user_name} by all"
          when /group/
            "revoke #{rights.join(',')} on database from group #{resource.group_name} by all"
          else
            fail "Class #{self.class} not supported for revoke"
          end
        end

        def grant(rights)
          case self.class.to_s
          when /role/
            "grant #{rights.join(',')} on database to role #{resource.role_name} "
          when /user/
            "grant #{rights.join(',')} on database to user #{resource.user_name} "
          when /group/
            "grant #{rights.join(',')} on database to group #{resource.group_name} "
          else
            fail "Class #{self.class} not supported for grant"
          end
        end

        def after_apply
          when_primary do
            db2(grant(granted_rights), :instance => resource.instance_name, :database => resource.database_name) if granted_rights.any?
            db2(revoke(revoked_rights), :instance => resource.instance_name, :database => resource.database_name) if revoked_rights.any?
          end
        end

        # Docs
        module ClassMethods
          def translate_to_array(properties)
            return [] if properties.nil?

            grants = []
            grants << 'BINDADD' if properties['BINDADDAUTH'] == 'Y'
            grants << 'CONNECT' if properties['CONNECTAUTH'] == 'Y'
            grants << 'CREATETAB' if properties['CREATETABAUTH'] == 'Y'
            grants << 'DBADM' if properties['DBADMAUTH'] == 'Y'
            grants << 'CREATE_EXTERNAL_ROUTINE' if properties['EXTERNALROUTINEAUTH'] == 'Y'
            grants << 'IMPLICIT_SCHEMA' if properties['IMPLSCHEMAAUTH'] == 'Y'
            grants << 'LOAD' if properties['LOADAUTH'] == 'Y'
            grants << 'CREATE_NOT_FENCED_ROUTINE' if properties['NOFENCEAUTH'] == 'Y'
            grants << 'QUIESCE_CONNECT' if properties['QUIESCECONNECTAUTH'] == 'Y'
            grants << 'LIBRARYADM' if properties['LIBRARYADMAUTH'] == 'Y'
            grants << 'SECADM' if properties['SECURITYADMAUTH'] == 'Y'
            grants << 'SQLADM' if properties['SQLADMAUTH'] == 'Y'
            grants << 'WLMADM' if properties['WLMADMAUTH'] == 'Y'
            grants << 'EXPLAIN' if properties['EXPLAINAUTH'] == 'Y'
            grants << 'DATAACCESS' if properties['DATAACCESSAUTH'] == 'Y'
            grants << 'ACCESSCTRL' if properties['ACCESSCTRLAUTH'] == 'Y'
            grants << 'CREATE_SECURE_OBJECT' if properties['CREATESECUREAUTH'] == 'Y'
            grants
          end

          def translate_to_resource(raw_resource)
            database     = raw_resource[:database]
            instance     = raw_resource[:instance]
            case self.to_s
            when /role/
              name = raw_resource['ROLENAME']
              grantee_type = 'R'
            when /user/
              name = raw_resource['USERNAME']
              grantee_type = 'U'
            when /group/
              name = raw_resource['GROUPNAME']
              grantee_type = 'G'
            else
              fail "Class #{self} not supported for roles"
            end

            @grants ||= {}
            @grants[instance] ||= {}
            @grants[instance][database] ||= sql('select * from syscat.dbauth', :instance => instance, :database => database, :parse => true)
            found_grants = @grants[instance][database].find do |record|
              record['GRANTEE'].strip == name.strip && record['GRANTEETYPE'] == grantee_type
            end
            translate_to_array(found_grants)
          end
        end
      end
    end
  end
end
