# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
module PuppetX
  module EnterpriseModules
    module Db2
      # Add documentation TODO
      module RoleProperty
        def self.included(parent)
          parent.send(:include, ::EasyType)
          parent.send(:include, ::EasyType::Mungers::Upcase)
          parent.send(:include, PuppetX::EnterpriseModules::Db2::Commands)
          parent.send(:include, PuppetX::EnterpriseModules::Db2::Property)
          parent.extend(ClassMethods)
        end

        def change_to_s(_from, _to)
          return_value = []
          return_value << "revoked the #{revoked_roles.join(',')} role(s)" if revoked_roles.any?
          return_value << "granted the #{granted_roles.join(',')} role(s)" if granted_roles.any?
          return_value.join(' and ')
        end

        def revoked_roles
          current_roles - value
        end

        def granted_roles
          value - current_roles
        end

        def current_roles
          return [] if provider.roles == :absent

          provider.roles
        end

        def revoke(roles)
          case self.class.to_s
          when /role/
            "revoke role #{roles.join(',')} from role #{resource.role_name}"
          when /user/
            "revoke role #{roles.join(',')} from user #{resource.user_name}"
          when /group/
            "revoke role #{roles.join(',')} from group #{resource.group_name}"
          else
            fail "Class #{self.classname} not supported for revoke"
          end
        end

        def grant(roles)
          case self.class.to_s
          when /role/
            "grant role #{roles.join(',')} to role #{resource.role_name} "
          when /user/
            "grant role #{roles.join(',')} to user #{resource.user_name} "
          when /group/
            "grant role #{roles.join(',')} to user #{resource.group_name} "
          else
            fail "Class #{self.classname} not supported for role"
          end
        end

        def after_apply
          when_primary do
            db2(grant(granted_roles), :instance => resource.instance_name, :database => resource.database_name) if granted_roles.any?
            db2(revoke(revoked_roles), :instance => resource.instance_name, :database => resource.database_name) if revoked_roles.any?
          end
        end

        # Docs
        module ClassMethods
          def translate_to_resource(raw_resource)
            database     = raw_resource[:database]
            instance     = raw_resource[:instance]
            case self.to_s
            when /role/
              name = raw_resource['ROLENAME']
              grantee_type = 'R'
            when /user/
              name = raw_resource['USERNAME']
              grantee_type = 'U'
            when /group/
              name = raw_resource['GROUPNAME']
              grantee_type = 'G'
            else
              fail "Class #{self} not supported for roles"
            end

            @roles ||= {}
            @roles[instance] ||= {}
            @roles[instance][database] ||= sql('select r.rolename, a.grantee, a.granteetype from SYSCAT.ROLEAUTH a, SYSCAT.ROLES r where r.roleid = a.roleid', :instance => instance, :database => database, :parse => true)
            selection = @roles[instance][database].select do |record|
              record['GRANTEE'].strip == name.strip && record['GRANTEETYPE'] == grantee_type
            end
            selection.collect { |e| e['ROLENAME'].strip }
          end
        end
      end
    end
  end
end
